from commandList import commandQueue
from PyQt4.QtCore import QObject, pyqtSignal
class _contentList(list):
  def sort(self):
    list.sort(self,key=lambda tile:tile.flag)
class GameObject(QObject):
  """
  This is the class which all objects in the game must subclass. This
  enables us to keep a list of all game objects.

  This enables a list of actors to be easily be tracked for collision
  detection - this should also be useful for debugging, and probably for
  purposes I have not yet thought of.
  """
  ItemAdded = pyqtSignal(str)
  Changed   = pyqtSignal()
  _allObjects = []
  def __getitem__(self,i):
    return self.contents[i]
  def getAll(self):
    """Returns all GameObjects"""
    return self._allObjects
  def getAllSameClass(self):
    """Returns all GameObjects decended from the same type as self"""
    return [x for x in self._allObjects
            if isinstance(x, type(self))]

  def _custAssert(self,(expected, got, input), e_args):
    return ' '.join(("You tried to input a", `input`, ':',
                     `type(got)`, ',', `got`, 'into ',
                     self.name, "when expecting", `expected`+'.',
                     "Original message:", `e_args`))
  def addContent(self,content):
    try:
      info = (GameObject, content, "Item to be added to "+self.name)
      assert isinstance(info[1], info[0])
      self.contents.append(content)
    except Exception, e:
      e.args = self._custAssert(info, e.args),
      raise
  def __init__(self, position=(0,0),
                     heading=(0, 0),
                     size=(-1,-1),
                     stats={},
                     contents=[],
                     name="Unknown Object",
                     image='',
                     flag=0):
    QObject.__init__(self)

    self.commands = commandQueue()
    try:
      # On some platforms there is no support for python beyond 2.5.4
      # ABC's were only implemented in python 2.6 so we'll fall back to
      # integers if we can't use Number - the base class for int and float etc
      from numbers import Number
    except:
      Number = int
    try:
      self.flag = flag
      
      # Check position is a tuple
      info = (tuple, position, "position")
      assert isinstance(info[1], info[0])
      info = (Number, position[0], "position_x")
      assert isinstance(info[1], info[0])
      info = (Number, position[1], "position_y")
      assert isinstance(info[1], info[0])
      self.position = position

      # Check heading is a tuple
      info = (tuple, heading, "heading")
      assert isinstance(info[1], info[0])
      info = (Number, heading[0], "heading_speed")
      assert isinstance(info[1], info[0])
      info = (Number, heading[1], "heading_XY_angle")
      assert isinstance(info[1], info[0])
      self.heading = heading

      # Check size is a tuple
      info = (tuple, size, "size")
      assert isinstance(info[1], info[0])
      self.size = size

      # Check name is a string
      info = (str, name, "name")
      assert isinstance(info[1], info[0])
      self.name = name

      # Check stats is a dictionary
      info = (dict, stats, "stats")
      assert isinstance(info[1], info[0])
      self.stats = stats.copy()

      # Check contents is a list
      info = (list, contents, "contents")
      assert isinstance(info[1], info[0])
      for obj in contents:
          info = (GameObject, obj, "object in contents")
          assert isinstance(info[1], info[0])
      def _sort():
        pass
      self.contents = _contentList(contents[:])
      #self.contents.sort = _sort
      self._allObjects.append(self)
      self._allObjects[0].ItemAdded.emit(self.name)

      # Check image is a string
      info = (str, image, "image")
      assert isinstance(info[1], info[0])
      self.image = image
    except Exception, e:
      e.args = self._custAssert(info, e.args),
      raise
